@@ -789,7 +789,10 @@ nasl_eregmatch(lex_ctxt* lexic)
     }
   
   if(nasl_regexec(&re, string, (size_t)NS, subs, 0) != 0)
+   {
+    nasl_regfree(&re);
     return NULL;
+   }
       
   retc = alloc_tree_cell(0, NULL);
   retc->type = DYN_ARRAY;
@@ -827,7 +830,7 @@ tree_cell* nasl_substr(lex_ctxt* lexic)
 #define MAX_INT (~(1 << (sizeof(int) * 8 - 1)))
 #endif
   i2 = get_int_var_by_num(lexic, 2, MAX_INT);
-  if (i2 > sz1) i2 = sz1-1;
+  if (i2 >= sz1) i2 = sz1-1;
 
   if (s1 == NULL || i1 < 0)
     {
@@ -876,7 +879,7 @@ tree_cell* nasl_insstr(lex_ctxt* lexic)
       return NULL;
     }
 
-  if (i1 > sz1)
+  if (i1 >= sz1)
     {
       nasl_perror(lexic, "insstr: cannot insert string2 after end of string1\n");
       return NULL;
@@ -1246,7 +1249,7 @@ nasl_str_replace(lex_ctxt * lexic)
   s = emalloc(1);
   sz2 = 0;
   n = 0;
-  for (i1 = i2 = 0; i1 < sz_a - sz_b; )
+  for (i1 = i2 = 0; i1 <= sz_a - sz_b; )
     {
       c = (char*)nasl_memmem(a + i1, sz_a - i1, b, sz_b);
       if(c == NULL)
